#NoTrayIcon
#RequireAdmin
#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Version=Beta
#AutoIt3Wrapper_Outfile=A3Service_Wrapper_X86.exe
#AutoIt3Wrapper_Outfile_x64=A3Service_Wrapper_X64.exe
#AutoIt3Wrapper_UseUpx=n
#AutoIt3Wrapper_Compile_Both=y
#AutoIt3Wrapper_UseX64=y
#AutoIt3Wrapper_Change2CUI=n
#AutoIt3Wrapper_Res_Comment=Written by Bitboy for Autoit.de, Administrator.de
#AutoIt3Wrapper_Res_Fileversion=0.2.0.0
#AutoIt3Wrapper_Tidy_Stop_OnError=n
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****

;================================================================
; Program:			A3Service Wrapper
; Function:			Runs external program as Service
; AutoIt Version:	3.3.9.21
; Program Version:	0.2 Alpha don't use in production environment!
; Created:			08.10.2013
; Last modified:	/
;
; Tested Systems:	Win 7 x64, Server2008 R2
;
; Author:			Bitboy
;
; Sources from:		arcker, http://www.autoitscript.com/forum/topic/80201-service-udf-v3-run-your-exe-as-service/page-15 (the older Version of Services.au3 see post #284)
;
;================================================================

#include <Winapi.au3>
#include <Services.au3>

#include <EventLog.au3>

#include <ButtonConstants.au3>
#include <ComboConstants.au3>
#include <EditConstants.au3>
#include <GUIConstantsEx.au3>
#include <StaticConstants.au3>
#include <WindowsConstants.au3>
#include <GUIComboBox.au3>

Global $sServiceName = StringReplace(@ScriptName, ".exe", "")
Global $iAppPID = ""
Global $hApp = ""
Global $iAppExit = ""
Global $debug = False

#region Service Functions

Func _main($iArg, $sArgs)
	Dim $sAppName = RegRead("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "Executable")
	Dim $sAppArgs = RegRead("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "Parameter")
	$aSuccessExitCodes = StringSplit(RegRead("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "SuccessExitCodes"), ",")
	$iAppRestart = RegRead("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "AutoRestart")

	If RegRead("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "Debug") = 1 Then $debug = True

	Dim $i
	Dim $bExpectedExit = False
	Dim $bAppRestart = False
	$bServiceRunning = True
	_WriteLog("AppName: " & $sAppName)
	_WriteLog("Arguments: " & $sAppArgs)
	_Service_ReportStatus($SERVICE_RUNNING, $NO_ERROR, 0)

	While $bServiceRunning ; REQUIRED  ( dont change variable name ) ; there are several ways to find that service have to be stoped - $Running flag in loop is the first method
		#region --> insert your running code here
		If Not ProcessExists($iAppPID) Then
			If $iAppPID <> "" Then
				$iAppExit = _ProcessGetExitCode($hApp)
				_WriteLog("Exit Code: " & $iAppExit)
				_ProcessCloseHandle($hApp)

				;Check if Termination was expected
				For $i = 1 To $aSuccessExitCodes[0]
					If $aSuccessExitCodes[$i] = $iAppExit Then
						_WriteLog("Termination expected with Exit Code: " & $iAppExit)
						_CreateEvent(4, 30, $sAppName & " terminated expected with Exit Code: " & $iAppExit)
						$bExpectedExit = True
						ExitLoop
					EndIf
				Next

				If $bExpectedExit = False Then
					_WriteLog("Termination NOT expected with Exit Code: " & $iAppExit)
					_CreateEvent(1, 60, $sAppName & " terminated unexpected with Exit Code: " & $iAppExit)
				EndIf

				;Check if Restart is configured
				;0 = Never, 1 = Always, 2 = only on error, 3 = only on success
				If $iAppRestart = 1 Or ($iAppRestart = 2 And $bExpectedExit = False) Or ($iAppRestart = 3 And $bExpectedExit = True) Then
					$iAppPID = "" ; <- This is used to initiate restart in the next loop
					$bAppRestart = True
					_WriteLog("Auto Restart initialized.")
					_CreateEvent(4, 15, $sAppName & " is going to restart as configured.")
				Else
					_WriteLog("Auto Restart not configured. Service is going to stop.")
					_CreateEvent(4, 40, $sAppName & " was terminated. Service is going to stop.")
					;Stop Service
					_Service_Stop($sServiceName)
				EndIf

			Else
				If $sAppArgs = "" Then
					$iAppPID = Run($sAppName)
				Else
					$iAppPID = Run($sAppName & " " & $sAppArgs)
				EndIf
				$hApp = _ProcessGetHandle($iAppPID)
				$bExpectedExit = False
				_WriteLog("PID: " & $iAppPID)
				_WriteLog("Handle: " & $hApp)
				If $bAppRestart Then
					_CreateEvent(4, 20, $sAppName & " is restarted with PID: " & $iAppPID)
				Else
					_CreateEvent(4, 10, $sAppName & " is started with PID: " & $iAppPID)
				EndIf
			EndIf
		EndIf
		#endregion --> insert your running code here
		_Sleep(100)
	WEnd
	If ProcessClose($iAppPID) Then _CreateEvent(4, 40, $sAppName & " was terminated because of service stop.")
	_Service_ReportStatus($SERVICE_STOP_PENDING, $NO_ERROR, 3000)
	Sleep(2000)
	_Service_ReportStatus($SERVICE_STOPPED, $NO_ERROR, 1000)
EndFunc   ;==>_main

; Return handle of given PID
Func _ProcessGetHandle($iPID)
	Local Const $PROCESS_QUERY_INFORMATION = 0x0400
	Local $avRET = DllCall($hKernel32_DLL, "ptr", "OpenProcess", "int", $PROCESS_QUERY_INFORMATION, "int", 0, "int", $iPID)
	If @error Then
		Return SetError(1, 0, 0)
	Else
		Return $avRET[0]
	EndIf
EndFunc   ;==>_ProcessGetHandle

; Close process handle
Func _ProcessCloseHandle($hProc)
	Local $avRET = DllCall($hKernel32_DLL, "int", "CloseHandle", "ptr", $hProc)
	If @error Then
		Return SetError(1, 0, 0)
	Else
		Return 1
	EndIf
EndFunc   ;==>_ProcessCloseHandle

; Get process exit code from handle
Func _ProcessGetExitCode($hProc)
	Local $t_ExitCode = DllStructCreate("int")
	Local $avRET = DllCall($hKernel32_DLL, "int", "GetExitCodeProcess", "ptr", $hProc, "ptr", DllStructGetPtr($t_ExitCode))
	If @error Then
		Return SetError(1, 0, 0)
	Else
		Return DllStructGetData($t_ExitCode, 1)
	EndIf
EndFunc   ;==>_ProcessGetExitCode

Func logprint($Data)
	_WriteLog($Data)
EndFunc   ;==>logprint

Func _WriteLog($Data)
	If $debug Then
		FileWriteLine(@ScriptDir & "\Service_" & $sServiceName & "_Log.txt", @MDAY & "." & @MON & "." & @YEAR & " " & @HOUR & ":" & @MIN & ":" & @SEC & @TAB & $Data)
	EndIf
EndFunc   ;==>_WriteLog

Func _CreateEvent($iEventType, $iEventID, $sEventMessage)
	Local $hEventLog, $aData[1] = [0]
	$hEventLog = _EventLog__Open("", "A3SW " & $sServiceName)
	_EventLog__Report($hEventLog, $iEventType, 0, $iEventID, @UserName, $sEventMessage, $aData)
	_EventLog__Close($hEventLog)
EndFunc   ;==>_CreateEvent

Func _Sleep($delay)
	Local $result = DllCall($hKernel32_DLL, "none", "Sleep", "dword", $delay)
EndFunc   ;==>_Sleep

Func _exit()
	_WriteLog("Service is stopping.")
	ProcessClose($iAppPID)
EndFunc   ;==>_exit

#endregion Service Functions

;=================================
;===        Script Start       ===
;=================================
If $cmdline[0] > 0 Then
	If $cmdline[1] = "service" Then ;Script is called with Parameter -> Service Mode
		_Service_init($sServiceName)
	EndIf
Else
	WinSetState(@ScriptFullPath, "", @SW_HIDE) ; <- Hide Console Window created by cui compiling
	;Show GUI
	#region ### START Koda GUI section ### Form=c:\users\sumannk\desktop\services\frmmain.kxf
	$frmMain = GUICreate("A3Service Wrapper", 457, 290, -1, -1, $GUI_SS_DEFAULT_GUI, $WS_EX_TOOLWINDOW)
	$lblprogname = GUICtrlCreateLabel("Programm to start:", 8, 10, 89, 17)
	$txtprogname = GUICtrlCreateInput("", 120, 8, 297, 21)
	$cmdprogname = GUICtrlCreateButton("", 424, 6, 25, 25, $BS_ICON)
	GUICtrlSetImage(-1, "C:\Windows\System32\shell32.dll", -4, 0)
	$lblprogparam = GUICtrlCreateLabel("Parameter:", 8, 84, 55, 17)
	$txtprogarguments = GUICtrlCreateInput("Param1 ""Param With Space"" Param3", 120, 80, 329, 21)
	$lblexitcodes = GUICtrlCreateLabel("Succesful exit codes:", 8, 116, 105, 17)
	$txtexitcodes = GUICtrlCreateInput("0", 120, 112, 329, 21)
	$cmddelservice = GUICtrlCreateButton("Delete Service", 344, 256, 105, 25)
	$cmdcreateservice = GUICtrlCreateButton("Create Service", 232, 256, 105, 25)
	$txtservicedesc = GUICtrlCreateEdit("", 120, 144, 329, 57, BitOR($ES_AUTOVSCROLL, $ES_WANTRETURN, $WS_VSCROLL))
	GUICtrlSetData(-1, "This is a A3Service Wrapper Service Description!")
	$lblservicedesc = GUICtrlCreateLabel("Service Description:", 8, 144, 99, 17)
	$lblstarttype = GUICtrlCreateLabel("Start Type:", 8, 220, 56, 17)
	$cbostarttype = GUICtrlCreateCombo("", 120, 216, 329, 25, BitOR($CBS_DROPDOWNLIST, $CBS_AUTOHSCROLL))
	GUICtrlSetData(-1, "Automatic|Manual|Disabled")
	;$ckbDebug = GUICtrlCreateCheckbox("Enable Debug Mode", 8, 264, 169, 17)
	$optrestartalways = GUICtrlCreateRadio("Restart always", 120, 56, 89, 17)
	$optrestartonerror = GUICtrlCreateRadio("Restart if program exits unexpected", 224, 32, 193, 17)
	$optrestartonsuccess = GUICtrlCreateRadio("Restart if program exits successfully", 224, 56, 185, 17)
	$optrestartnever = GUICtrlCreateRadio("Restart never", 120, 32, 89, 17)
	GUICtrlSetState(-1, $GUI_CHECKED)
	$lblrestartoptions = GUICtrlCreateLabel("Restart options:", 8, 32, 78, 17)
	GUISetState(@SW_SHOW)
	#endregion ### END Koda GUI section ###
	_GUICtrlComboBox_SetCurSel($cbostarttype, 0)


	While 1
		$nMsg = GUIGetMsg()
		Switch $nMsg
			Case $GUI_EVENT_CLOSE
				Exit
			Case $cmddelservice
				;Delete Service
				RemoveService()
			Case $cmdcreateservice
				;Create Service
				InstallService()
			Case $cmdprogname
				_SelectProgram()
		EndSwitch
	WEnd

EndIf

#region GUI Functions
Func InstallService()
	Dim $iAutoRestart = 0
	Dim $iDebug = 0
	Dim $iStartType = 0
	Dim $iAnswer = 0
	#RequireAdmin

	;Check Input
	If GUICtrlRead($txtprogname) = "" Or Not FileExists(GUICtrlRead($txtprogname)) Then
		MsgBox(16, "Error", "Select a program to run.")
		Return 1
	EndIf

	If GUICtrlRead($optrestartnever) = $GUI_CHECKED Then
		$iAutoRestart = 0
	ElseIf GUICtrlRead($optrestartalways) = $GUI_CHECKED Then
		$iAutoRestart = 1
	ElseIf GUICtrlRead($optrestartonerror) = $GUI_CHECKED Then
		$iAutoRestart = 2
	ElseIf GUICtrlRead($optrestartonsuccess) = $GUI_CHECKED Then
		$iAutoRestart = 4
	EndIf

	#cs
	If GUICtrlRead($ckbDebug) = $GUI_CHECKED Then
		$iDebug = 1
	EndIf
	#ce

	Switch _GUICtrlComboBox_GetCurSel($cbostarttype)
		Case 0
			$iStartType = $SERVICE_AUTO_START ;Automatic
		Case 1
			$iStartType = $SERVICE_DEMAND_START ;Manual
		Case 2
			$iStartType = $SERVICE_DISABLED ;Disabled
	EndSwitch

	$ServiceType = $SERVICE_WIN32_OWN_PROCESS

	_Service_Create($sServiceName, $sServiceName, $ServiceType, $iStartType, $SERVICE_ERROR_SEVERE, '"' & @ScriptFullPath & '" service')

	If @error Then
		MsgBox(16, "A3Service Wrapper", "Error creating service.")
		Return 1
	EndIf

	;Add registry Parameters
	RegWrite("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "Executable", "REG_SZ", '"' & GUICtrlRead($txtprogname) & '"')
	RegWrite("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "Parameter", "REG_SZ", GUICtrlRead($txtprogarguments))
	RegWrite("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "SuccessExitCodes", "REG_SZ", GUICtrlRead($txtexitcodes))
	RegWrite("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "AutoRestart", "REG_DWORD", $iAutoRestart)
	;RegWrite("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName & "\Parameters", "Debug", "REG_DWORD", $iDebug)

	RegWrite("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\" & $sServiceName, "Description", "REG_SZ", GUICtrlRead($txtservicedesc))

	;RegKey used for EventViewer Source Entry
	RegWrite("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\eventlog\Application\A3SW " & $sServiceName)

	$iAnswer = MsgBox(68, "A3Service Wrapper", "Service created sucessfully." & @CRLF & "Would you like to start the service now?")
	If $iAnswer = 6 Then
		_Service_Start($sServiceName)
	EndIf
	Return 0
EndFunc   ;==>InstallService

Func RemoveService()
	_Service_Stop($sServiceName)
	_Service_Delete($sServiceName)

	;Delete RegKey used by EventViewer Source
	RegDelete("HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\eventlog\Application\A3SW " & $sServiceName)

	If Not @error Then
		MsgBox(64, "A3Service Wrapper", "Service removed sucessfully.")
	Else
		MsgBox(16, "A3Service Wrapper", "Error removing service.")
	EndIf
	Return 0
EndFunc   ;==>RemoveService


Func _SelectProgram()
	Dim $sSelected
	$sSelected = FileOpenDialog("Select Program to start as Service", "", "(*.exe)|(*.bat)", 1)
	If $sSelected <> "" Then
		GUICtrlSetData($txtprogname, $sSelected)
	EndIf

EndFunc   ;==>_SelectProgram

#endregion GUI Functions

